#!/usr/local/bin/python
#vim ts=4:et
# Tar[.gz] extraction utility
#   Usage: python extract.py <archive.[tar][gz]> [outpath] [[file1] [file2] [file3] ...]
import gzip
import os.path
import sys
import tarfile
import shutil

# Exit error codes
EXIT_SUCCESS = 0
ERROR_NO_ARCHIVE_SPECIFIED = 1
ERROR_ARCHIVE_DOES_NOT_EXIST = 2
ERROR_INVALID_ARCHIVE = 3

# Debugging (set to 1 to enable, 0 to disable)
DEBUG = 0

# Die with error code
def die(code):
    if DEBUG: print code
    sys.exit(code)

# Parse command line arguments
def parse_args(args):
    args.pop(0)

    # Get archive name
    try: archive = args.pop(0)
    except: die(ERROR_NO_ARCHIVE_SPECIFIED)
    
    #FIXME: Workaround since not able to address files in external source (CD) using consistent drive name - hence using file system name
    #Change to FS name
    os.system('map -u >a map.txt')
    os.system('map >a map.txt')
    alias=archive.split(':')[0]
    map=open('map.txt','r')
    fsfound=0
    for lines in map.readlines():
       if (lines.find(alias) != -1):
          fsname=lines.split(':')[0].lstrip(' ').rstrip(' ')
          fsfound=1
          break
    if (not fsfound):
        print 'File System for alias not found'
        die(ERROR_ARCHIVE_DOES_NOT_EXIST)
    archive=fsname+':'+archive.split(':')[1]

    # Check that archive exists
    if not os.path.exists(archive): die(ERROR_ARCHIVE_DOES_NOT_EXIST)

    # Check that archive specified is a tar.gz archive
    type = ''
    if tarfile.is_tarfile(archive):
        type = 'tar'
    # Check if archive is a plain .gz archive
    elif archive[-3:].lower() == '.gz':
        try:
            f = gzip.open(archive)
            f.read(1)
            f.close()
            type = 'gzip'
        except IOError: die(ERROR_INVALID_ARCHIVE)
    else: die(ERROR_INVALID_ARCHIVE)

    # Get output path if any
    try: path = args.pop(0)
    except: path = ''
    try: os.makedirs(path)
    except: pass

    # Get files to extract
    files = args

    return [type, archive, path, files]

# Extract specified archive to specified path or current directory
def extract_archive(archive, path, files=[]):
    if(1):
	extract_all, cannonical_dirs, cannonical_files = True, [], []
        if len(files): # Cannonicalize user supplied dirs and files
            extract_all = False
            cannonical_dirs = [ d.lstrip('/').rstrip('/')+'/' for d in filter( lambda x: x.endswith('/'), files ) ]
            cannonical_files = [ f.lstrip('/') for f in filter( lambda x: not x.endswith('/'), files ) ]

        # Open archive and loop through entries
        for i in range(10):
            try:
                tarobj = tarfile.open(archive)
                break
            except: print 'tar open failed'
        for entry in tarobj: 
            tar_entry_name = entry.name.lstrip('/')
            if entry.isdir(): tar_entry_name = tar_entry_name.rstrip('/') + '/'

            if extract_all \
                or tar_entry_name in cannonical_files \
                or reduce( lambda x,y: x or y, map( lambda x: tar_entry_name.startswith(x), cannonical_dirs + [ f+'/' for f in cannonical_files ]), False):
		#FIXME: The above line is a workaround for dir input from drivers.xml file not ending with / for recursive behavior
		#       Once fixed in Catalog.xml, uncomment below line.
		#or reduce( lambda x,y: x or y, map( lambda x: tar_entry_name.startswith(x), cannonical_dirs), False):
                if DEBUG: print 'Extracting ' + entry.name
                tarobj.extract(entry, path)

        tarobj.close()
    else:
        die(ERROR_INVALID_ARCHIVE)

def extract_gz(archive, path):
    try:
        # Figure out output filename
        outfile = os.path.join(path, os.path.basename(archive)[:-3])

        # Extract file
        if DEBUG: print 'Extracting ' + os.path.basename(outfile)
	shutil.copyfileobj( gzip.open(archive), open(outfile, 'wb'))
    except:
        die(ERROR_INVALID_ARCHIVE)

# Entry point
if __name__ == '__main__':
    # Parse command line arguments
    [type, archive, path, files] = parse_args(sys.argv)

    if type == 'tar':
        extract_archive(archive, path, files)
    elif type == 'gzip':
        extract_gz(archive, path)

    die(EXIT_SUCCESS)
